from PyQt5 import Qt
from PyQt5.QtCore import pyqtSignal

from settings import settings

class WDMWebEngineProfile(Qt.QWebEngineProfile):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.settings().setAttribute(Qt.QWebEngineSettings.PluginsEnabled, True)
        self.setHttpAcceptLanguage("sv")


class WDMWebEnginePage(Qt.QWebEnginePage):
    def acceptNavigationRequest(self, url:Qt.QUrl, navigation_type, isMainFrame:bool):
        ret = super().acceptNavigationRequest(url, navigation_type, isMainFrame)
        return True #ret

    #def createWindow(self, web_window_type):
    #    return self


class WDMDownload(Qt.QObject):
    downloadFinished = pyqtSignal(object)

    def __init__(self, parent, download_item:Qt.QWebEngineDownloadItem):
        super().__init__(parent)
        self.download_item = download_item
        self.download_item.finished.connect(self.handleFinished)
        self.download_item.accept()

    def handleFinished(self):
        self.downloadFinished.emit(self.download_item)


class WDMBrowser(Qt.QWebEngineView):
    loggedIn = pyqtSignal()
    txtFileDownloaded = pyqtSignal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.downloads = []
        self.setPage(WDMWebEnginePage(WDMWebEngineProfile("wdm", self), self))
        self.page().profile().downloadRequested.connect(self.handleDownloadRequested)

        self.register_func = None
        self.load(Qt.QUrl("https://www.dhlmultishipping.com"))
        self.loadFinished.connect(self.handleLoadFinished)

    def createWindow(self, web_window_type):
        window = WDMBrowser(self)
        window.show()
        return window

    def handleDownloadRequested(self, download_item:Qt.QWebEngineDownloadItem):
        download = WDMDownload(self, download_item)
        download.downloadFinished.connect(self.handleDownloadFinished)
        self.downloads.append(download)

    def handleDownloadFinished(self, download_item):
        if download_item.path().endswith('.txt'):
            self.txtFileDownloaded.emit(download_item.path())
        else:
            Qt.QDesktopServices.openUrl(Qt.QUrl(download_item.path()))

    def handleLoadFinished(self, ok = True):
        url = self.url().toString()
        print(url)
        if url.startswith('https://www.dhlmultishipping.com/login/#/to/'):
            Qt.QTimer.singleShot(1000, self.login)
        elif url.startswith('https://www.dhlmultishipping.com/np/main#/home/dhl'):
            self.loggedIn.emit()
            self.load(Qt.QUrl('https://www.dhlmultishipping.com/transnet/register.do?method=view'))
        elif url.startswith('https://www.dhlmultishipping.com/transnet/register.do?method=view'):
            if self.register_func:
                func = self.register_func
                self.register_func = None
                func()

    def login(self):
        self.page().runJavaScript('jQuery("#username").val("{}");null;'.format(settings.get('dhl_username', '')))
        self.page().runJavaScript('jQuery("#domain").val("{}");null;'.format(settings.get('dhl_company_id', '')))
        self.page().runJavaScript('jQuery("#password").val("{}");null;'.format(settings.get('dhl_password', '')))
        if settings.get('dhl_password', ''):
            self.page().runJavaScript('jQuery("#bt-login").click();null;')

    def fillIn(self, order):
        self.order = order
        self.page().runJavaScript(
            'jQuery("select[name=templatePK] > option:contains(\'{}\')")[0].value'.format(
                settings.get('dhl_template', '')
            ),
            self.setTemplate,
        )

    def setTemplate(self, result):
        self.page().runJavaScript(
            'jQuery("select[name=templatePK]").val("{}");null;'.format(
                result
            )
        )
        self.register_func = self.enterStart
        self.page().runJavaScript('jQuery("#refreshButton_actionButton").click();')

    def enterStart(self):
        self.page().runJavaScript('jQuery("#orderNo").val("{}");null;'.format(self.order['number']))
        self.page().runJavaScript('jQuery("#consigneeReference").val("{}");null;'.format(self.order['number']))
        self.page().runJavaScript('jQuery("#consignee_mode_other").click();null;')
        Qt.QTimer.singleShot(500, self.enterDetails)


    def enterDetails(self):
        self.page().runJavaScript('jQuery("[name=\'consignee_temp.id\']").val("{}");null;'.format(self.order['number']))
        self.page().runJavaScript('jQuery("[name=\'consignee_temp.name\']").val("{}");null;'.format(
            self.order['shipping']['company'] or  "{} {}".format(
                self.order['shipping']['first_name'], self.order['shipping']['last_name']
            )
        ))
        self.page().runJavaScript('jQuery("[name=\'consignee_temp.addressline[0]\']").val("{}");null;'.format(self.order['shipping']['address_1']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.addressline[1]\']").val("{}");null;'.format(self.order['shipping']['address_2']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.postcode\']").val("{}");null;'.format(self.order['shipping']['postcode']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.city\']").val("{}");null;'.format(self.order['shipping']['city']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee.countrycode\']").val("{}");null;'.format(self.order['shipping']['country']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.contactPerson\']").val("{} {}");null;'.format(self.order['shipping']['first_name'], self.order['shipping']['last_name']))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.phone\']").val("{}");null;'.format(self.order['shipping'].get('phone',  self.order['billing']['phone'])))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.mobile\']").val("{}");null;'.format(self.order['shipping'].get('phone', self.order['billing']['phone'])))
        self.page().runJavaScript(
            'jQuery("[name=\'consignee_temp.email\']").val("{}");null;'.format(self.order['shipping'].get('email',  self.order['billing']['email'])))
        # Packages
        self.page().runJavaScript(
            'jQuery("[name=\'goodsItemList.goodsItems[0].noOfPackages\']").val("{}");null;'.format(1))
        self.page().runJavaScript(
            'jQuery("[name=\'goodsItemList.goodsItems[0].weight\']").val("{}");null;'.format(settings.get('dhl_weight', '1')))

        self.page().runJavaScript('jQuery("#refreshButton_actionButton").click();')
