from PyQt5 import Qt
from gui.browser import WDMBrowser
from gui.woo import WooTableView
from settings import settings, set_setting

SHIPMENT_ACTION = '{"endpoint":"orders/{id}/notes", "context": {"note": "DHL {shipment_number}", "customer_note": true}}'

class WDMMainWindow(Qt.QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.initUI()

    def initUI(self):
        self.mainVBoxLayout = Qt.QVBoxLayout()

        # DHL Settings
        self.dhlsSettingsLabel = Qt.QLabel(self.tr("DHL Multi Shipping settings - "))
        self.dhlsSettingsUsernameLabel = Qt.QLabel(self.tr("Username:"))
        self.dhlsSettingsUsername = Qt.QLineEdit(settings.get('dhl_username', ''))
        self.dhlsSettingsUsername.textChanged.connect(set_setting('dhl_username'))
        self.dhlsSettingsCompanyIDLabel = Qt.QLabel(self.tr("Company-ID:"))
        self.dhlsSettingsCompanyID = Qt.QLineEdit(settings.get('dhl_company_id', ''))
        self.dhlsSettingsCompanyID.textChanged.connect(set_setting('dhl_company_id'))
        self.dhlsSettingsPasswordLabel = Qt.QLabel(self.tr("Password:"))
        self.dhlsSettingsPassword = Qt.QLineEdit(settings.get('dhl_password', ''))
        self.dhlsSettingsPassword.textChanged.connect(set_setting('dhl_password'))
        self.dhlSettingsTemplateLabel = Qt.QLabel(self.tr("Template name:"))
        self.dhlSettingsTemplate = Qt.QLineEdit(settings.get('dhl_template', 'DHL SERVICEPOINT'))
        self.dhlSettingsTemplate.textChanged.connect(set_setting('dhl_template'))

        self.dhlSettingsWeightLabel = Qt.QLabel(self.tr("Default weight:"))
        self.dhlSettingsWeight = Qt.QLineEdit(settings.get('dhl_weight', '1'))
        self.dhlSettingsWeight.textChanged.connect(set_setting('dhl_weight'))

        # DHL Layout
        self.dhlSettingsLayout = Qt.QHBoxLayout()
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsLabel)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsUsernameLabel)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsUsername)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsCompanyIDLabel)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsCompanyID)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsPasswordLabel)
        self.dhlSettingsLayout.addWidget(self.dhlsSettingsPassword)
        self.dhlSettingsLayout.addWidget(self.dhlSettingsTemplateLabel)
        self.dhlSettingsLayout.addWidget(self.dhlSettingsTemplate)
        self.dhlSettingsLayout.addWidget(self.dhlSettingsWeightLabel)
        self.dhlSettingsLayout.addWidget(self.dhlSettingsWeight)
        self.mainVBoxLayout.addLayout(self.dhlSettingsLayout)

        # Woo Settings
        self.wooSettingsLabel = Qt.QLabel(self.tr("Woo settings - "))
        self.wooSettingsAddressLabel = Qt.QLabel(self.tr("Address:"))
        self.wooSettingsAddress = Qt.QLineEdit(settings.get('woo_address', ''))
        self.wooSettingsAddress.textChanged.connect(set_setting('woo_address'))
        self.wooSettingsAddress.setPlaceholderText(self.tr("https://www.example.com/"))
        self.wooSettingsConsumerKeyLabel = Qt.QLabel(self.tr("Consumer key:"))
        self.wooSettingsConsumerKey = Qt.QLineEdit(settings.get('woo_consumer_key', ''))
        self.wooSettingsConsumerKey.textChanged.connect(set_setting('woo_consumer_key'))
        self.wooSettingsConsumerSecretLabel = Qt.QLabel(self.tr("Consumer secret:"))
        self.wooSettingsConsumerSecret = Qt.QLineEdit(settings.get('woo_consumer_secret', ''))
        self.wooSettingsConsumerSecret.textChanged.connect(set_setting('woo_consumer_secret'))
        self.wooSettingsShipmentLabel = Qt.QLabel(self.tr("Shipment action:"))
        self.wooSettingsShipment = Qt.QLineEdit(settings.get('woo_shipment', SHIPMENT_ACTION))
        self.wooSettingsShipment.textChanged.connect(set_setting('woo_shipment'))

        # Woo Layout
        self.wooSettingsLayout = Qt.QHBoxLayout()
        self.wooGetOrdersPushButton = Qt.QPushButton(self.tr("Get orders"))
        self.wooSettingsLayout.addWidget(self.wooSettingsLabel)
        self.wooSettingsLayout.addWidget(self.wooSettingsAddressLabel)
        self.wooSettingsLayout.addWidget(self.wooSettingsAddress)
        self.wooSettingsLayout.addWidget(self.wooSettingsConsumerKeyLabel)
        self.wooSettingsLayout.addWidget(self.wooSettingsConsumerKey)
        self.wooSettingsLayout.addWidget(self.wooSettingsConsumerSecretLabel)
        self.wooSettingsLayout.addWidget(self.wooSettingsConsumerSecret)
        self.wooSettingsLayout.addWidget(self.wooSettingsShipmentLabel)
        self.wooSettingsLayout.addWidget(self.wooSettingsShipment)
        self.wooSettingsLayout.addWidget(self.wooGetOrdersPushButton)
        self.mainVBoxLayout.addLayout(self.wooSettingsLayout)

        # Woo Table
        self.wooTableView = WooTableView(self)
        self.wooTableView.setMaximumHeight(200)
        self.wooGetOrdersPushButton.clicked.connect(self.wooTableView.model().update)
        self.mainVBoxLayout.addWidget(self.wooTableView)

        # The browser
        self.browser = WDMBrowser()
        self.wooTableView.fillIn.connect(self.browser.fillIn)
        self.browser.loggedIn.connect(self.wooTableView.model().update)
        self.browser.txtFileDownloaded.connect(self.wooTableView.model().txtFileDownloaded)
        self.mainVBoxLayout.addWidget(self.browser)

        # The last things
        self.setWindowTitle('Woo DHL Multi Shipping')
        self.setLayout(self.mainVBoxLayout)

        self.showMaximized()
        self.show()
        #self.setGeometry(300, 300, 350, 300)
